import { select } from 'd3';
import { viz } from './viz';
import { data } from '@vspatel927/nyc-water';
import { observeResize } from '@curran/8a596d3b83374cceaa320d0e00127d98';

export const main = (container, { state, setState }) => {
  const dimensions = observeResize({
    state,
    setState,
    container,
  });

  if (dimensions === null) return;

  const { width, height } = dimensions;

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height);

  viz(svg, {
    data,
    xValue: (d) => d.NYC_Consumption,
    xAxisLabelText: 'Water Consumption (1: 1 million)',
    xAxisLabelOffset: 38,
    yValue: (d) => d.New_York_City_Population,
    yAxisLabelText: 'NYC Population',
    yAxisLabelOffset: 17,
    innerRectFill: '#E8E8E8',
    circleRadius: 153 / 20,
    circleOpacity: 629 / 1000,
    marginTop: 20,
    marginBottom: 47,
    marginLeft: 86,
    marginRight: 20,
    width,
    height,
  });
};
