import { scaleLinear, axisLeft, extent, select } from 'd3';
import {
  xAxis,
  yAxis,
  xAxisLabel,
  yAxisLabel,
  axes,
} from '@curran/8a596d3b83374cceaa320d0e00127d98';

export const viz = (
  svg,
  {
    data,
    xValue,
    xAxisLabelText,
    xAxisLabelOffset,
    yValue,
    yAxisLabelText,
    yAxisLabelOffset,
    marginLeft,
    marginTop,
    marginRight,
    marginBottom,
    width,
    height,
    innerRectFill,
    circleRadius,
    circleOpacity,
  },
) => {
  const xScale = scaleLinear(extent(data, xValue), [
    marginLeft,
    width - marginRight,
  ]);
  const yScale = scaleLinear(extent(data, yValue), [
    height - marginBottom,
    marginTop,
  ]);

  const innerWidth = width - marginLeft - marginRight;
  const innerHeight = height - marginTop - marginBottom;

  // Define a custom water droplet shape using SVG path commands
  const dropletPath =
    'M8 16C3.58 16 0 12.42 0 8c0-5.52 8-16 8-16s8 10.48 8 16c0 4.42-3.58 8-8 8z';

  const tooltip = select('body')
    .append('div')
    .attr('class', 'tooltip')
    .style('opacity', 0);

  svg
    .selectAll('rect')
    .data([null])
    .join('rect')
    .attr('x', marginLeft)
    .attr('y', marginTop)
    .attr('width', innerWidth)
    .attr('height', innerHeight)
    .attr('fill', 'url(#image-pattern)');

  // Axes
  xAxis(svg, { height, marginBottom, xScale });
  yAxis(svg, { marginLeft, yScale });

  // Axis Labels
  xAxisLabel(svg, {
    xAxisLabelText,
    width,
    height,
    marginBottom,
    xAxisLabelOffset,
  });
  yAxisLabel(svg, {
    yAxisLabelText,
    yAxisLabelOffset,
    height,
  });

  const defs = svg.append('defs');

  const gradient = defs
    .append('linearGradient')
    .attr('id', 'water-gradient')
    .attr('x1', '0%')
    .attr('y1', '0%')
    .attr('x2', '0%')
    .attr('y2', '100%');

  gradient
    .append('stop')
    .attr('class', 'stop1')
    .attr('offset', '0%')
    .attr('stop-color', '#aad3df')
    .attr('stop-opacity', 1);

  gradient
    .append('stop')
    .attr('class', 'stop2')
    .attr('offset', '100%')
    .attr('stop-color', '#2b8a9c')
    .attr('stop-opacity', 1);

  //Droplets
  svg
    .selectAll('path')
    .data(data.reverse())
    .enter()
    .append('path')
    .attr('d', dropletPath) // Use the custom droplet shape
    .attr(
      'transform',
      (d) =>
        `translate(${xScale(xValue(d))},${-20}) scale(0) rotate(0, 8, 8)`,
    )
    .on('mouseover', function (event, d) {
      select(this).attr('opacity', '.1');
      tooltip
        .transition()
        .duration(1000)
        .style('opacity', 0.9);
      tooltip
        .html(
          `Year: ${d['Year']} <br> Consumption: ${d['NYC_Consumption'] * 1000000} <br> Population: ${d['New_York_City_Population']} <br> Per Capita: ${d['Per_Capita']}`,
        )
        .style('position', 'fixed')
        .style('left', event.pageX + 10 + 'px')
        .style('top', event.pageY + 10 + 'px')
        .style('background', '#ffffff');
    })
    // Hide tooltip on mouseout
    .on('mouseout', function () {
      select(this).attr('opacity', '1');
      tooltip.style('opacity', 0);
    })

    // Start with 0 scale for animation
    .style('fill', 'url(#water-gradient)') // Apply water-like gradient

    .transition()
    .duration(1000)
    .delay((d, i) => i * 200)
    .attr(
      'transform',
      (d) =>
        `translate(${xScale(xValue(d))},${yScale(yValue(d))}) scale(1) rotate(0, 8, 8)`,
    ); // Scale to 1 to appear fully

  svg
    .append('defs')
    .append('pattern')
    .attr('id', 'image-pattern') // Unique ID for the pattern
    .attr('patternUnits', 'userSpaceOnUse') // Define the pattern units
    .attr('width', width) // Width of the pattern
    .attr('height', height) // Height of the pattern
    .append('image')
    .attr(
      'xlink:href',
      'https://s7d2.scene7.com/is/image/TWCNews/NY_NYC_Cloudy_Rain_AP110405031573_MarkLennihan',
    ) // Link to your image
    .attr('width', width) // Width of the image
    .attr('height', height) // Height of the image
    .attr('opacity', 0.5);
};
